#![allow(dead_code)]

mod tag;
mod util;
mod runtime;
mod interface; pub use interface::*;

fn check<T>(expected:T, actual:T) where T:std::fmt::Display, T:std::cmp::PartialEq
{
    print!(" ({}) ", if expected == actual { " " } else { "!" });
    println!("'{}' = '{}'", expected, actual);
}

pub fn test() {
    //unsafe { runtime::test(); }

    let mut sch = Schema::new();
    sch.assign("x", integer());
    sch.bind(0x10);
}

#[cfg(test)]
mod tests {
    #[test]
    fn boolean_initialize()
    {
        let value = crate::Boolean::new();
        assert_eq!(false, value.get(), "Boolean::new() produces value of false.");
    }

    #[test]
    fn boolean_with()
    {
        assert_eq!(false, crate::Boolean::with(false).get(), "Boolean::with(false) produces value of false.");
        assert_eq!(true, crate::Boolean::with(true).get(), "Boolean::with(true) produces value of true.");
    }

    #[test]
    fn boolean_set_get()
    {
        let mut value = crate::Boolean::new();
        value.set(true);
        assert_eq!(true, value.get(), "Boolean.set(true) produces value of true.");
        value.set(false);
        assert_eq!(false, value.get(), "Boolean.set(false) produces value of false.");
    }

    #[test]
    fn natural_initialize()
    {
        let value = crate::Natural::new();
        assert_eq!(0, value.get(), "Natural::new() produces value of 0.");
    }

    #[test]
    fn natural_set_get()
    {
        let mut value = crate::Natural::new();
        value.set(12);
        assert_eq!(12, value.get(), "Natural.set(12) produces value of 12.");
        value.set(38695);
        assert_eq!(38695, value.get(), "Natural.set(38695) produces value of 38695.");
    }

    #[test]
    fn integer_initialize()
    {
        let value = crate::Integer::new();
        assert_eq!(0, value.get(), "Integer::new() produces value of 0.");
    }

    #[test]
    fn integer_set_get()
    {
        let mut value = crate::Integer::new();
        value.set(-273);
        assert_eq!(-273, value.get(), "Integer.set(-273) produces value of -273.");
        value.set(100);
        assert_eq!(100, value.get(), "Integer.set(100) produces value of 100.");
    }

    #[test]
    fn block_initialize()
    {
        let value = crate::Block::new(8);
        assert_eq!(8, value.length(), "Block::new(8) has length of 8.");
        assert_eq!(vec![0;8], value.get(), "Block::new(8) has value of 0.");
    }

    #[test]
    fn block_set_get()
    {
        let mut value = crate::Block::new(4);
        value.set(vec![1, 2, 3, 4]);
        assert_eq!(vec![1, 2, 3, 4], value.get(), "Block::set([1,2,3,4]) has value of [1,2,3,4].");
    }

    #[test]
    fn sequence_initialize()
    {
        let value = crate::Sequence::new();
        assert_eq!(0, value.length(), "Sequence::new() has length 0.");
    }

    #[test]
    fn sequence_set_get_raw()
    {
        let mut value = crate::Sequence::new();
        value.set_raw(vec![0, 1, 2, 3, 2, 1]);
        assert_eq!(6, value.length(), "Sequence.set_raw([0,1,2,3,2,1]) has length 6.");
        assert_eq!(vec![0,1,2,3,2,1], value.get_raw(), "Sequence.set_raw([0,1,2,3,2,1]) produces value [0,1,2,3,2,1].");
    }

    #[test]
    fn sequence_set_get_str()
    {
        let mut value = crate::Sequence::new();
        value.set("hello");
        assert_eq!(5, value.length(), "Sequence.set(hello) has length 5.");
        assert_eq!(String::from("hello"), value.get(), "Sequence.set(hello) produces value hello.");
    }
}
